@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4 -*-
 /* This file is part of the KDE project
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright (C) 2000, 2001 David Faure <david@mandrakesoft.com>
+   Copyright (C) 2000-2004 David Faure <faure@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -30,9 +30,12 @@ using namespace std;
 #include <kurl.h>
 #include <kservice.h>
 #include <koGlobal.h>
+#include <koPageLayout.h>
+#include <koffice_export.h>
 
 class QDomElement;
 class QDomDocument;
+class QXmlSimpleReader;
 
 class KoStore;
 class KoMainWindow;
@@ -42,16 +45,17 @@ class KoDocumentChild;
 class KoView;
 class KoDocumentInfo;
 class DCOPObject;
-
+class KoOasisStyles;
+class KoXmlWriter;
 
 /**
- *  The KOffice document class
+ *  The %KOffice document class
  *
- *  This class provides some functionality each KOffice document should have.
+ *  This class provides some functionality each %KOffice document should have.
  *
- *  @short The KOffice document class
+ *  @short The %KOffice document class
  */
-class KoDocument : public KParts::ReadWritePart
+class KOFFICECORE_EXPORT KoDocument : public KParts::ReadWritePart
 {
     Q_OBJECT
     Q_PROPERTY( QCString dcopObjectId READ dcopObjectId)
@@ -105,7 +109,7 @@ public:
      * of @ref #element is of interest here. The method searches first in the
      * @ref KActionCollection of the first view and then in the KActionCollection of this
      * document.
-     * This allows KOffice applications to define actions in both the view and the document.
+     * This allows %KOffice applications to define actions in both the view and the document.
      * They should only define view-actions (like zooming and stuff) in the view.
      * Every action which changes the document should be defined in the document.
      *
@@ -140,7 +144,7 @@ public:
      * Call this instead of openURL() to implement KoMainWindow's
      * File --> Import feature.
      *
-     * Note: This will call openURL(). To differentiate this from an ordinary
+     * @note This will call openURL(). To differentiate this from an ordinary
      *       Open operation (in any reimplementation of openURL() or openFile())
      *       call @ref isImporting().
      */
@@ -152,14 +156,15 @@ public:
      * KParts::ReadWritePart::saveAs() to implement KoMainWindow's
      * File --> Export feature.
      *
-     * Note: This will call KoDocument::saveAs(). To differentiate this
+     * @note This will call KoDocument::saveAs(). To differentiate this
      *       from an ordinary Save operation (in any reimplementation of
      *       saveFile()) call @ref isExporting().
      */
     bool exp0rt( const KURL &url );
 
     /**
-     * Sets whether the document can be edited or is read only.
+     * @brief Sets whether the document can be edited or is read only.
+     *
      * This recursively applied to all child documents and
      * @ref KoView::updateReadWrite is called for every attached
      * view.
@@ -167,7 +172,7 @@ public:
     virtual void setReadWrite( bool readwrite = true );
 
     /**
-     * Used by KoApplication, when no document exists yet.
+     * @brief Used by KoApplication, and by KoMainWindow, when no document exists yet.
      *
      * With the help of @p instance or @ref KApplication::instance() this
      * method figures out which .desktop file matches this application. In this
@@ -176,21 +181,58 @@ public:
      * @see KService
      * @see KDesktopFile
      */
-    static QCString readNativeFormatMimeType( KInstance *instance = 0L );
+    static QCString readNativeFormatMimeType( KInstance *instance = 0 );
 
-    static KService::Ptr readNativeService( KInstance *instance = 0L );
+    /**
+     * Used by KoMainWindow, when no document exists yet.
+     *
+     * With the help of @p instance or @ref KApplication::instance() this
+     * method figures out which .desktop file matches this application. In this
+     * file it searches for the "X-KDE-ExtraNativeMimeTypes" entry and returns it.
+     *
+     * @see KService
+     * @see KDesktopFile
+     */
+    static QStringList readExtraNativeMimeTypes( KInstance *instance = 0 );
+
+    /**
+     * setup the XML reader, so that we don't have to duplicate the code.
+     */
+    static void setupXmlReader( QXmlSimpleReader& reader, bool namespaceProcessing = false );
 
     /**
      * To be preferred when a document exists. It is fast when calling
      * it multiple times since it caches the result that @ref #readNativeFormatMimeType
      * delivers.
+     * This comes from the X-KDE-NativeMimeType key in the .desktop file
      * You do NOT have to reimplement this (it is only virtual for kounavail).
      */
     virtual QCString nativeFormatMimeType() const;
 
-    KService::Ptr nativeService();
+    /**
+     * Returns the OASIS OpenDocument mimetype of the document, if supported
+     * This comes from the X-KDE-NativeOasisMimeType key in the .desktop file
+     */
+    QCString nativeOasisMimeType() const;
 
-    enum { SaveAsKOffice1dot1 = 1, SaveAsDirectoryStore = 2 };
+    /// Checks whether a given mimetype can be handled natively.
+    bool isNativeFormat( const QCString& mimetype ) const;
+
+    /// Returns a list of the mimetypes considered "native", i.e. which can
+    /// be saved by KoDocument without a filter, in *addition* to the main one
+    virtual QStringList extraNativeMimeTypes() const;
+
+    /// Enum values used by specialOutputFlag - note that it's a bitfield for supportedSpecialFormats
+    enum { SaveAsKOffice1dot1 = 1,
+           SaveAsDirectoryStore = 2
+           // bitfield! next value is 4
+         };
+
+    /**
+     * Return the set of SupportedSpecialFormats that the application wants to
+     * offer in the "Save" file dialog.
+     */
+    virtual int supportedSpecialFormats() const;
 
     /**
      * Returns the actual mimetype of the document
@@ -198,7 +240,16 @@ public:
     QCString mimeType() const;
 
     /**
-     * Set the format in which the document should be saved.
+     * @brief Sets the mime type for the document.
+     *
+     * When choosing "save as" this is also the mime type
+     * selected by default.
+     */
+    void setMimeType( const QCString & mimeType );
+
+    /**
+     * @brief Set the format in which the document should be saved.
+     *
      * This is called on loading, and in "save as", so you shouldn't
      * have to call it.
      * @param specialOutputFlag is for "save as older version" etc.
@@ -231,6 +282,30 @@ public:
      */
     void setErrorMessage( const QString& errMsg );
 
+    /**
+     * Return the last error message. Usually KoDocument takes care of
+     * showing it; this method is mostly provided for non-interactive use.
+     * @since 1.4
+     */
+    QString errorMessage() const;
+
+    /**
+     * Show the last error message in a message box.
+     * The dialog box will mention a saving problem.
+     * Note that save/saveFile takes care of doing it.
+     * @since 1.4
+     */
+    void showSavingErrorDialog();
+
+    /**
+     * Show the last error message in a message box.
+     * The dialog box will mention a loading problem.
+     * openURL/openFile takes care of doing it, but not loadNativeFormat itself,
+     * so this is often called after loadNativeFormat returned false.
+     * @since 1.4
+     */
+    void showLoadingErrorDialog();
+
     /**
      *  Create a new view for the document.
      */
@@ -296,8 +371,8 @@ public:
                                   KoView *view = 0L, double zoomX = 1.0, double zoomY = 1.0 );
 
     /**
-     * Generates a preview picture of the document
-     * (to be used in the File Dialog).
+     * @brief Generates a preview picture of the document
+     * @note The preview is used in the File Dialog and also to create the Thumbnail
      */
     virtual QPixmap generatePreview( const QSize& size );
 
@@ -331,7 +406,7 @@ public:
     /**
      *  Paints the data itself. Normally called by @ref paintEverthing. It does not
      *  paint the children.
-     *  It's this method that KOffice Parts have to implement.
+     *  It's this method that %KOffice Parts have to implement.
      *
      *  @param painter     The painter object into that should be drawn.
      *  @param rect        The rect that should be used in the painter object.
@@ -350,24 +425,20 @@ public:
     bool checkAutoSaveFile();
 
     /**
-     *  Initializes an empty document (display the template dialog!).
-     *  You have to overload this method to initialize all your document variables.
-     */
-    virtual bool initDoc() = 0; // BCI: pass a QWidget* parentWidget, to pass to KoTemplateChooseDia
-
-    // Who calls initDoc?
-    enum InitDocFlags { InitDocAppStarting, InitDocFileNew, InitDocFileClose, InitDocEmbedded };
-    /**
-     * Return some flags for initDoc (BCI: this should be merged with initDoc)
+     * This setting indicates who is calling initDoc.
      * Usually the app will want to
      * - show the template dialog with 'everything' if InitDocAppStarting, InitDocFileClose or InitDocEmbedded
      * - show the template dialog with 'templates only' if InitDocFileNew
+     * - create an empty document with default settings if InitDocEmpty
      */
-    InitDocFlags initDocFlags() const;
+    enum InitDocFlags { InitDocAppStarting, InitDocFileNew, InitDocFileClose, InitDocEmbedded, InitDocEmpty };
+
     /**
-     * For KoMainWindow only - don't use!
+     * Initializes an empty document (display the template dialog!).
+     * You have to overload this method to initialize all your document variables.
+     * @param flags see InitDocFlags
      */
-    void setInitDocFlags( InitDocFlags flags );
+    virtual bool initDoc(InitDocFlags flags, QWidget* parentWidget=0) = 0;
 
     /**
      *  Sets the modified flag on the document. This means that it has
@@ -388,7 +459,9 @@ public:
     virtual bool isEmpty() const { return m_bEmpty; }
 
     /**
-     *  Sets the document to empty. Used after loading a template
+     *  @brief Sets the document to empty.
+     *
+     *  Used after loading a template
      *  (which is not empty, but not the user's input).
      *
      *  @see #isEmpty
@@ -396,21 +469,29 @@ public:
     virtual void setEmpty() { m_bEmpty = true; }
 
     /**
-     *  Loads a document from a store.
+     *  @brief Loads a document from a store.
+     *
      *  You should never have to reimplement.
      *  @param url An internal url, like tar:/1/2
      */
     virtual bool loadFromStore( KoStore* store, const QString& url );
 
     /**
-     *  Saves a document to a store.
+     *  @brief Loads an OASIS document from a store.
+     *  This is used for both the main document and embedded objects.
+     */
+    virtual bool loadOasisFromStore( KoStore* store );
+
+    /**
+     *  @brief Saves a document to a store.
+     *
      *  You should not have to reimplement this - but call it in @ref #saveChildren.
      */
     virtual bool saveToStore( KoStore* store, const QString& path );
 
     /**
-     *  Reimplement this method to load the contents of your KOffice document,
-     *  from the XML document.
+     *  Reimplement this method to load the contents of your %KOffice document,
+     *  from the XML document. This is for the pre-Oasis file format (maindoc.xml).
      *
      *  You are supposed to use the QDomDocument. The QIODevice is provided only
      *  for the cases where some pre-processing is needed, like kpresenter's kprconverter.
@@ -419,7 +500,22 @@ public:
     virtual bool loadXML( QIODevice *, const QDomDocument & doc ) = 0;
 
     /**
-     *  Reimplement this to save the contents of the KOffice document into
+     *  Reimplement this method to load the contents of your %KOffice document,
+     *  from the XML document ("content.xml"). The styles have been parsed already,
+     *  you can find them in the oasisStyles parameter. The store can be used
+     *  to load images and embedded documents.
+     */
+    virtual bool loadOasis( const QDomDocument & doc, KoOasisStyles& oasisStyles,
+                            const QDomDocument & settings, KoStore* store ) = 0;
+
+    /**
+     *  Reimplement this method to save the contents of your %KOffice document,
+     *  using the OASIS format.
+     */
+    virtual bool saveOasis( KoStore* store, KoXmlWriter* manifestWriter ) = 0;
+
+    /**
+     *  Reimplement this to save the contents of the %KOffice document into
      *  a QDomDocument. The framework takes care of saving it to the store.
      */
     virtual QDomDocument saveXML();
@@ -429,11 +525,31 @@ public:
      *  including processing instruction, complete DOCTYPE tag (with systemId and publicId), and root element.
      *  @param tagName the name of the tag for the root element
      *  @param version the DTD version (usually the application's version).
+     *  @deprecated use createOasisXmlWriter instead
      */
     QDomDocument createDomDocument( const QString& tagName, const QString& version ) const;
 
     /**
-     *  Return a correctly created QDomDocument for a KOffice document,
+     *  Return an XML writer for saving Oasis XML into the device @p dev,
+     *  including the XML processing instruction,
+     *  and the root element with all its namespaces.
+     *  You can add more namespaces afterwards with addAttribute.
+     *
+     *  @param dev the device into which the XML will be written.
+     *  @param rootElementName the tag name of the root element.
+     *     This is either office:document, office:document-content,
+     *     office:document-styles, office:document-meta or office:document-settings
+     *  @return the KoXmlWriter instance. It becomes owned by the caller, which
+     *  must delete it at some point.
+     *
+     * Once done with writing the contents of the root element, you
+     * will need to call endElement(); endDocument(); before destroying the KoXmlWriter.
+     * @note OASIS-specific
+     */
+    static KoXmlWriter* createOasisXmlWriter( QIODevice* dev, const char* rootElementName );
+
+    /**
+     *  Return a correctly created QDomDocument for an old (1.3-style) %KOffice document,
      *  including processing instruction, complete DOCTYPE tag (with systemId and publicId), and root element.
      *  This static method can be used e.g. by filters.
      *  @param appName the app's instance name, e.g. kword, kspread, kpresenter etc.
@@ -473,7 +589,7 @@ public:
     /**
      * Checks whether the document is currently in the process of autosaving
      */
-    bool isAutosaving();
+    bool isAutosaving() const;
 
     /**
      * Set whether the next openURL call should check for an auto-saved file
@@ -489,6 +605,12 @@ public:
      */
     void setAutoErrorHandlingEnabled( bool b );
 
+    /**
+     * Checks whether error message boxes should be shown.
+     * @since 1.3.1
+     */
+    bool isAutoErrorHandlingEnabled() const;
+
     /**
      * Retrieve the default value for autosave in seconds.
      * Called by the applications to use the correct default in their config
@@ -550,8 +672,9 @@ public:
     static QPtrList<KoDocument> *documentList() { return s_documentList; }
 
     /**
-     * Return a DCOP interface for this document
-     * KOffice parts are strongly recommended to reimplement this method,
+     * @brief Return a DCOP interface for this document
+     *
+     * %KOffice parts are strongly recommended to reimplement this method,
      * so that their DCOP interface provides more functionality than the basic KoDocumentIface
      */
     virtual DCOPObject * dcopObject();
@@ -566,7 +689,7 @@ public:
      */
     void emitProgress( int value ) { emit sigProgress( value ); }
 
-    bool isInOperation();
+    bool isInOperation() const;
     virtual void emitBeginOperation();
     virtual void emitEndOperation();
 
@@ -574,7 +697,7 @@ public:
      * Return true if url() is a real filename, false if url() is
      * an internal url in the store, like "tar:/..."
      */
-    virtual bool isStoredExtern();
+    virtual bool isStoredExtern() const;
 
     /**
      * @return the page layout associated with this document (margins, paper, etc).
@@ -595,21 +718,28 @@ public:
     /**
      * Returns true if this document or any of its internal child documents are modified.
      */
-    bool isModified();
+    bool isModified() const;
+
+    /**
+     * Returns true during loading (openURL can be asynchronous)
+     */
+    bool isLoading() const;
 
     int queryCloseExternalChildren();
     int queryCloseDia();
 
     /**
-     * Set when we do not want to save external children when saving our 'main' doc.
+     * @brief Set when we do not want to save external children when saving our 'main' doc.
+     *
      * This makes it possible to save 'main' doc + all its internal children first, then
      * go on to save external children. Typically used by query close.
      * Use:
+     * @code
      *      doc->setDoNotSaveExtDoc();
      *      doc->save();    // saves doc and its internal children,
      *                            //also calls saveExternalChildren() which sets setDoNotSaveExtDoc(false)
      *      doc->saveExternalChildren();
-     *
+     * @endcode
      */
     void setDoNotSaveExtDoc( bool on = true );
 
@@ -640,7 +770,7 @@ public:
 
     /**
      * Sets the document URL to empty URL
-     * KParts doesn't allow this, but KOffice apps have e.g. templates
+     * KParts doesn't allow this, but %KOffice apps have e.g. templates
      * After using loadNativeFormat on a template, one wants
      * to set the url to KURL()
      */
@@ -656,7 +786,7 @@ public:
      */
     bool storeInternal() const;
 
-    bool hasExternURL();
+    bool hasExternURL() const;
 
     /**
      * Sets the document URL to @p url
@@ -668,17 +798,27 @@ public:
      * _Only_ use these functions to restore m_file (in KoMainWindow) after a
      * failed save (remember to use setURL() to restore the URL as well).
      *
-     * Do _not_ use these functions for any other purpose.
+     * @warning Do _not_ use these functions for any other purpose.
      *
      * @internal
      */
     QString &file() { return m_file; }
 
     /**
+     * _Only_ use these functions to restore m_file (in KoMainWindow) after a
+     * failed save (remember to use setURL() to restore the URL as well).
+     *
+     * @warning Do _not_ use these functions for any other purpose.
+     *
      * @internal
      */
     void setFile( const QString &file ) { m_file = file; }
 
+    /**
+     * @internal (public for KoMainWindow)
+     */
+    void setMimeTypeAfterLoading( const QString& mimeType );
+
 signals:
     /**
      * This signal is emitted when a direct or indirect child document changes
@@ -753,11 +893,14 @@ protected:
     bool isExporting() const;
 
     /**
+     *  OLD XML method. For OASIS just call KoDocumentChild::loadOasisDocument
+     *  after KoDocumentChild::loadOasis.
+     *
      *  You need to overload this function if your document may contain
      *  embedded documents. This function is called to load embedded documents.
      *
      *  An example implementation may look like this:
-     *  <PRE>
+     *  @code
      *  QPtrListIterator<KoDocumentChild> it( children() );
      *  for( ; it.current(); ++it )
      *  {
@@ -767,7 +910,7 @@ protected:
      *    }
      *  }
      *  return true;
-     *  </PRE>
+     *  @endcode
      */
     virtual bool loadChildren( KoStore* );
 
@@ -778,6 +921,14 @@ protected:
      */
     virtual bool saveChildren( KoStore* store );
 
+    /**
+     *  Saves all internal children (only!), to the store, using the OASIS format.
+     *  This is called automatically during saveNativeFormat.
+     *  @see saveExternalChildren if you have external children.
+     *  Returns true on success.
+     */
+    virtual bool saveChildrenOasis( KoStore* store, KoXmlWriter* manifestWriter );
+
     /**
      *  Overload this function if you have to load additional files
      *  from a store. This function is called after @ref #loadXML
@@ -823,7 +974,7 @@ protected:
      *  Returns true on success.
      * @see #saveChildren for internal children
      */
-    bool saveExternalChildren(); // BCI: make this virtual
+    virtual bool saveExternalChildren();
 
 private slots:
     void slotChildChanged( KoChild *c );
@@ -832,7 +983,13 @@ private slots:
     void slotStarted( KIO::Job* );
 
 private:
-    void savePreview( KoStore* store );
+    static KService::Ptr readNativeService( KInstance *instance );
+    KService::Ptr nativeService();
+
+    bool oldLoadAndParse( KoStore* store, const QString& filename, QDomDocument& doc );
+    bool loadNativeFormatFromStore( const QString& file );
+    bool savePreview( KoStore* store );
+    bool saveOasisPreview( KoStore* store, KoXmlWriter* manifestWriter );
 
     class Private;
     Private *d;
